/*
 * This header file was generated by ./256colres.pl
 */
/* $XTermId: 256colres.h,v 1.11 2007/06/12 00:44:23 tom Exp $ */

#ifndef included_256colres_h
#define included_256colres_h

COLOR_RES("16",	screen.Acolors[16],	DFT_COLOR("rgb:00/00/00")),
COLOR_RES("17",	screen.Acolors[17],	DFT_COLOR("rgb:00/00/5f")),
COLOR_RES("18",	screen.Acolors[18],	DFT_COLOR("rgb:00/00/87")),
COLOR_RES("19",	screen.Acolors[19],	DFT_COLOR("rgb:00/00/af")),
COLOR_RES("20",	screen.Acolors[20],	DFT_COLOR("rgb:00/00/d7")),
COLOR_RES("21",	screen.Acolors[21],	DFT_COLOR("rgb:00/00/ff")),
COLOR_RES("22",	screen.Acolors[22],	DFT_COLOR("rgb:00/5f/00")),
COLOR_RES("23",	screen.Acolors[23],	DFT_COLOR("rgb:00/5f/5f")),
COLOR_RES("24",	screen.Acolors[24],	DFT_COLOR("rgb:00/5f/87")),
COLOR_RES("25",	screen.Acolors[25],	DFT_COLOR("rgb:00/5f/af")),
COLOR_RES("26",	screen.Acolors[26],	DFT_COLOR("rgb:00/5f/d7")),
COLOR_RES("27",	screen.Acolors[27],	DFT_COLOR("rgb:00/5f/ff")),
COLOR_RES("28",	screen.Acolors[28],	DFT_COLOR("rgb:00/87/00")),
COLOR_RES("29",	screen.Acolors[29],	DFT_COLOR("rgb:00/87/5f")),
COLOR_RES("30",	screen.Acolors[30],	DFT_COLOR("rgb:00/87/87")),
COLOR_RES("31",	screen.Acolors[31],	DFT_COLOR("rgb:00/87/af")),
COLOR_RES("32",	screen.Acolors[32],	DFT_COLOR("rgb:00/87/d7")),
COLOR_RES("33",	screen.Acolors[33],	DFT_COLOR("rgb:00/87/ff")),
COLOR_RES("34",	screen.Acolors[34],	DFT_COLOR("rgb:00/af/00")),
COLOR_RES("35",	screen.Acolors[35],	DFT_COLOR("rgb:00/af/5f")),
COLOR_RES("36",	screen.Acolors[36],	DFT_COLOR("rgb:00/af/87")),
COLOR_RES("37",	screen.Acolors[37],	DFT_COLOR("rgb:00/af/af")),
COLOR_RES("38",	screen.Acolors[38],	DFT_COLOR("rgb:00/af/d7")),
COLOR_RES("39",	screen.Acolors[39],	DFT_COLOR("rgb:00/af/ff")),
COLOR_RES("40",	screen.Acolors[40],	DFT_COLOR("rgb:00/d7/00")),
COLOR_RES("41",	screen.Acolors[41],	DFT_COLOR("rgb:00/d7/5f")),
COLOR_RES("42",	screen.Acolors[42],	DFT_COLOR("rgb:00/d7/87")),
COLOR_RES("43",	screen.Acolors[43],	DFT_COLOR("rgb:00/d7/af")),
COLOR_RES("44",	screen.Acolors[44],	DFT_COLOR("rgb:00/d7/d7")),
COLOR_RES("45",	screen.Acolors[45],	DFT_COLOR("rgb:00/d7/ff")),
COLOR_RES("46",	screen.Acolors[46],	DFT_COLOR("rgb:00/ff/00")),
COLOR_RES("47",	screen.Acolors[47],	DFT_COLOR("rgb:00/ff/5f")),
COLOR_RES("48",	screen.Acolors[48],	DFT_COLOR("rgb:00/ff/87")),
COLOR_RES("49",	screen.Acolors[49],	DFT_COLOR("rgb:00/ff/af")),
COLOR_RES("50",	screen.Acolors[50],	DFT_COLOR("rgb:00/ff/d7")),
COLOR_RES("51",	screen.Acolors[51],	DFT_COLOR("rgb:00/ff/ff")),
COLOR_RES("52",	screen.Acolors[52],	DFT_COLOR("rgb:5f/00/00")),
COLOR_RES("53",	screen.Acolors[53],	DFT_COLOR("rgb:5f/00/5f")),
COLOR_RES("54",	screen.Acolors[54],	DFT_COLOR("rgb:5f/00/87")),
COLOR_RES("55",	screen.Acolors[55],	DFT_COLOR("rgb:5f/00/af")),
COLOR_RES("56",	screen.Acolors[56],	DFT_COLOR("rgb:5f/00/d7")),
COLOR_RES("57",	screen.Acolors[57],	DFT_COLOR("rgb:5f/00/ff")),
COLOR_RES("58",	screen.Acolors[58],	DFT_COLOR("rgb:5f/5f/00")),
COLOR_RES("59",	screen.Acolors[59],	DFT_COLOR("rgb:5f/5f/5f")),
COLOR_RES("60",	screen.Acolors[60],	DFT_COLOR("rgb:5f/5f/87")),
COLOR_RES("61",	screen.Acolors[61],	DFT_COLOR("rgb:5f/5f/af")),
COLOR_RES("62",	screen.Acolors[62],	DFT_COLOR("rgb:5f/5f/d7")),
COLOR_RES("63",	screen.Acolors[63],	DFT_COLOR("rgb:5f/5f/ff")),
COLOR_RES("64",	screen.Acolors[64],	DFT_COLOR("rgb:5f/87/00")),
COLOR_RES("65",	screen.Acolors[65],	DFT_COLOR("rgb:5f/87/5f")),
COLOR_RES("66",	screen.Acolors[66],	DFT_COLOR("rgb:5f/87/87")),
COLOR_RES("67",	screen.Acolors[67],	DFT_COLOR("rgb:5f/87/af")),
COLOR_RES("68",	screen.Acolors[68],	DFT_COLOR("rgb:5f/87/d7")),
COLOR_RES("69",	screen.Acolors[69],	DFT_COLOR("rgb:5f/87/ff")),
COLOR_RES("70",	screen.Acolors[70],	DFT_COLOR("rgb:5f/af/00")),
COLOR_RES("71",	screen.Acolors[71],	DFT_COLOR("rgb:5f/af/5f")),
COLOR_RES("72",	screen.Acolors[72],	DFT_COLOR("rgb:5f/af/87")),
COLOR_RES("73",	screen.Acolors[73],	DFT_COLOR("rgb:5f/af/af")),
COLOR_RES("74",	screen.Acolors[74],	DFT_COLOR("rgb:5f/af/d7")),
COLOR_RES("75",	screen.Acolors[75],	DFT_COLOR("rgb:5f/af/ff")),
COLOR_RES("76",	screen.Acolors[76],	DFT_COLOR("rgb:5f/d7/00")),
COLOR_RES("77",	screen.Acolors[77],	DFT_COLOR("rgb:5f/d7/5f")),
COLOR_RES("78",	screen.Acolors[78],	DFT_COLOR("rgb:5f/d7/87")),
COLOR_RES("79",	screen.Acolors[79],	DFT_COLOR("rgb:5f/d7/af")),
COLOR_RES("80",	screen.Acolors[80],	DFT_COLOR("rgb:5f/d7/d7")),
COLOR_RES("81",	screen.Acolors[81],	DFT_COLOR("rgb:5f/d7/ff")),
COLOR_RES("82",	screen.Acolors[82],	DFT_COLOR("rgb:5f/ff/00")),
COLOR_RES("83",	screen.Acolors[83],	DFT_COLOR("rgb:5f/ff/5f")),
COLOR_RES("84",	screen.Acolors[84],	DFT_COLOR("rgb:5f/ff/87")),
COLOR_RES("85",	screen.Acolors[85],	DFT_COLOR("rgb:5f/ff/af")),
COLOR_RES("86",	screen.Acolors[86],	DFT_COLOR("rgb:5f/ff/d7")),
COLOR_RES("87",	screen.Acolors[87],	DFT_COLOR("rgb:5f/ff/ff")),
COLOR_RES("88",	screen.Acolors[88],	DFT_COLOR("rgb:87/00/00")),
COLOR_RES("89",	screen.Acolors[89],	DFT_COLOR("rgb:87/00/5f")),
COLOR_RES("90",	screen.Acolors[90],	DFT_COLOR("rgb:87/00/87")),
COLOR_RES("91",	screen.Acolors[91],	DFT_COLOR("rgb:87/00/af")),
COLOR_RES("92",	screen.Acolors[92],	DFT_COLOR("rgb:87/00/d7")),
COLOR_RES("93",	screen.Acolors[93],	DFT_COLOR("rgb:87/00/ff")),
COLOR_RES("94",	screen.Acolors[94],	DFT_COLOR("rgb:87/5f/00")),
COLOR_RES("95",	screen.Acolors[95],	DFT_COLOR("rgb:87/5f/5f")),
COLOR_RES("96",	screen.Acolors[96],	DFT_COLOR("rgb:87/5f/87")),
COLOR_RES("97",	screen.Acolors[97],	DFT_COLOR("rgb:87/5f/af")),
COLOR_RES("98",	screen.Acolors[98],	DFT_COLOR("rgb:87/5f/d7")),
COLOR_RES("99",	screen.Acolors[99],	DFT_COLOR("rgb:87/5f/ff")),
COLOR_RES("100",	screen.Acolors[100],	DFT_COLOR("rgb:87/87/00")),
COLOR_RES("101",	screen.Acolors[101],	DFT_COLOR("rgb:87/87/5f")),
COLOR_RES("102",	screen.Acolors[102],	DFT_COLOR("rgb:87/87/87")),
COLOR_RES("103",	screen.Acolors[103],	DFT_COLOR("rgb:87/87/af")),
COLOR_RES("104",	screen.Acolors[104],	DFT_COLOR("rgb:87/87/d7")),
COLOR_RES("105",	screen.Acolors[105],	DFT_COLOR("rgb:87/87/ff")),
COLOR_RES("106",	screen.Acolors[106],	DFT_COLOR("rgb:87/af/00")),
COLOR_RES("107",	screen.Acolors[107],	DFT_COLOR("rgb:87/af/5f")),
COLOR_RES("108",	screen.Acolors[108],	DFT_COLOR("rgb:87/af/87")),
COLOR_RES("109",	screen.Acolors[109],	DFT_COLOR("rgb:87/af/af")),
COLOR_RES("110",	screen.Acolors[110],	DFT_COLOR("rgb:87/af/d7")),
COLOR_RES("111",	screen.Acolors[111],	DFT_COLOR("rgb:87/af/ff")),
COLOR_RES("112",	screen.Acolors[112],	DFT_COLOR("rgb:87/d7/00")),
COLOR_RES("113",	screen.Acolors[113],	DFT_COLOR("rgb:87/d7/5f")),
COLOR_RES("114",	screen.Acolors[114],	DFT_COLOR("rgb:87/d7/87")),
COLOR_RES("115",	screen.Acolors[115],	DFT_COLOR("rgb:87/d7/af")),
COLOR_RES("116",	screen.Acolors[116],	DFT_COLOR("rgb:87/d7/d7")),
COLOR_RES("117",	screen.Acolors[117],	DFT_COLOR("rgb:87/d7/ff")),
COLOR_RES("118",	screen.Acolors[118],	DFT_COLOR("rgb:87/ff/00")),
COLOR_RES("119",	screen.Acolors[119],	DFT_COLOR("rgb:87/ff/5f")),
COLOR_RES("120",	screen.Acolors[120],	DFT_COLOR("rgb:87/ff/87")),
COLOR_RES("121",	screen.Acolors[121],	DFT_COLOR("rgb:87/ff/af")),
COLOR_RES("122",	screen.Acolors[122],	DFT_COLOR("rgb:87/ff/d7")),
COLOR_RES("123",	screen.Acolors[123],	DFT_COLOR("rgb:87/ff/ff")),
COLOR_RES("124",	screen.Acolors[124],	DFT_COLOR("rgb:af/00/00")),
COLOR_RES("125",	screen.Acolors[125],	DFT_COLOR("rgb:af/00/5f")),
COLOR_RES("126",	screen.Acolors[126],	DFT_COLOR("rgb:af/00/87")),
COLOR_RES("127",	screen.Acolors[127],	DFT_COLOR("rgb:af/00/af")),
COLOR_RES("128",	screen.Acolors[128],	DFT_COLOR("rgb:af/00/d7")),
COLOR_RES("129",	screen.Acolors[129],	DFT_COLOR("rgb:af/00/ff")),
COLOR_RES("130",	screen.Acolors[130],	DFT_COLOR("rgb:af/5f/00")),
COLOR_RES("131",	screen.Acolors[131],	DFT_COLOR("rgb:af/5f/5f")),
COLOR_RES("132",	screen.Acolors[132],	DFT_COLOR("rgb:af/5f/87")),
COLOR_RES("133",	screen.Acolors[133],	DFT_COLOR("rgb:af/5f/af")),
COLOR_RES("134",	screen.Acolors[134],	DFT_COLOR("rgb:af/5f/d7")),
COLOR_RES("135",	screen.Acolors[135],	DFT_COLOR("rgb:af/5f/ff")),
COLOR_RES("136",	screen.Acolors[136],	DFT_COLOR("rgb:af/87/00")),
COLOR_RES("137",	screen.Acolors[137],	DFT_COLOR("rgb:af/87/5f")),
COLOR_RES("138",	screen.Acolors[138],	DFT_COLOR("rgb:af/87/87")),
COLOR_RES("139",	screen.Acolors[139],	DFT_COLOR("rgb:af/87/af")),
COLOR_RES("140",	screen.Acolors[140],	DFT_COLOR("rgb:af/87/d7")),
COLOR_RES("141",	screen.Acolors[141],	DFT_COLOR("rgb:af/87/ff")),
COLOR_RES("142",	screen.Acolors[142],	DFT_COLOR("rgb:af/af/00")),
COLOR_RES("143",	screen.Acolors[143],	DFT_COLOR("rgb:af/af/5f")),
COLOR_RES("144",	screen.Acolors[144],	DFT_COLOR("rgb:af/af/87")),
COLOR_RES("145",	screen.Acolors[145],	DFT_COLOR("rgb:af/af/af")),
COLOR_RES("146",	screen.Acolors[146],	DFT_COLOR("rgb:af/af/d7")),
COLOR_RES("147",	screen.Acolors[147],	DFT_COLOR("rgb:af/af/ff")),
COLOR_RES("148",	screen.Acolors[148],	DFT_COLOR("rgb:af/d7/00")),
COLOR_RES("149",	screen.Acolors[149],	DFT_COLOR("rgb:af/d7/5f")),
COLOR_RES("150",	screen.Acolors[150],	DFT_COLOR("rgb:af/d7/87")),
COLOR_RES("151",	screen.Acolors[151],	DFT_COLOR("rgb:af/d7/af")),
COLOR_RES("152",	screen.Acolors[152],	DFT_COLOR("rgb:af/d7/d7")),
COLOR_RES("153",	screen.Acolors[153],	DFT_COLOR("rgb:af/d7/ff")),
COLOR_RES("154",	screen.Acolors[154],	DFT_COLOR("rgb:af/ff/00")),
COLOR_RES("155",	screen.Acolors[155],	DFT_COLOR("rgb:af/ff/5f")),
COLOR_RES("156",	screen.Acolors[156],	DFT_COLOR("rgb:af/ff/87")),
COLOR_RES("157",	screen.Acolors[157],	DFT_COLOR("rgb:af/ff/af")),
COLOR_RES("158",	screen.Acolors[158],	DFT_COLOR("rgb:af/ff/d7")),
COLOR_RES("159",	screen.Acolors[159],	DFT_COLOR("rgb:af/ff/ff")),
COLOR_RES("160",	screen.Acolors[160],	DFT_COLOR("rgb:d7/00/00")),
COLOR_RES("161",	screen.Acolors[161],	DFT_COLOR("rgb:d7/00/5f")),
COLOR_RES("162",	screen.Acolors[162],	DFT_COLOR("rgb:d7/00/87")),
COLOR_RES("163",	screen.Acolors[163],	DFT_COLOR("rgb:d7/00/af")),
COLOR_RES("164",	screen.Acolors[164],	DFT_COLOR("rgb:d7/00/d7")),
COLOR_RES("165",	screen.Acolors[165],	DFT_COLOR("rgb:d7/00/ff")),
COLOR_RES("166",	screen.Acolors[166],	DFT_COLOR("rgb:d7/5f/00")),
COLOR_RES("167",	screen.Acolors[167],	DFT_COLOR("rgb:d7/5f/5f")),
COLOR_RES("168",	screen.Acolors[168],	DFT_COLOR("rgb:d7/5f/87")),
COLOR_RES("169",	screen.Acolors[169],	DFT_COLOR("rgb:d7/5f/af")),
COLOR_RES("170",	screen.Acolors[170],	DFT_COLOR("rgb:d7/5f/d7")),
COLOR_RES("171",	screen.Acolors[171],	DFT_COLOR("rgb:d7/5f/ff")),
COLOR_RES("172",	screen.Acolors[172],	DFT_COLOR("rgb:d7/87/00")),
COLOR_RES("173",	screen.Acolors[173],	DFT_COLOR("rgb:d7/87/5f")),
COLOR_RES("174",	screen.Acolors[174],	DFT_COLOR("rgb:d7/87/87")),
COLOR_RES("175",	screen.Acolors[175],	DFT_COLOR("rgb:d7/87/af")),
COLOR_RES("176",	screen.Acolors[176],	DFT_COLOR("rgb:d7/87/d7")),
COLOR_RES("177",	screen.Acolors[177],	DFT_COLOR("rgb:d7/87/ff")),
COLOR_RES("178",	screen.Acolors[178],	DFT_COLOR("rgb:d7/af/00")),
COLOR_RES("179",	screen.Acolors[179],	DFT_COLOR("rgb:d7/af/5f")),
COLOR_RES("180",	screen.Acolors[180],	DFT_COLOR("rgb:d7/af/87")),
COLOR_RES("181",	screen.Acolors[181],	DFT_COLOR("rgb:d7/af/af")),
COLOR_RES("182",	screen.Acolors[182],	DFT_COLOR("rgb:d7/af/d7")),
COLOR_RES("183",	screen.Acolors[183],	DFT_COLOR("rgb:d7/af/ff")),
COLOR_RES("184",	screen.Acolors[184],	DFT_COLOR("rgb:d7/d7/00")),
COLOR_RES("185",	screen.Acolors[185],	DFT_COLOR("rgb:d7/d7/5f")),
COLOR_RES("186",	screen.Acolors[186],	DFT_COLOR("rgb:d7/d7/87")),
COLOR_RES("187",	screen.Acolors[187],	DFT_COLOR("rgb:d7/d7/af")),
COLOR_RES("188",	screen.Acolors[188],	DFT_COLOR("rgb:d7/d7/d7")),
COLOR_RES("189",	screen.Acolors[189],	DFT_COLOR("rgb:d7/d7/ff")),
COLOR_RES("190",	screen.Acolors[190],	DFT_COLOR("rgb:d7/ff/00")),
COLOR_RES("191",	screen.Acolors[191],	DFT_COLOR("rgb:d7/ff/5f")),
COLOR_RES("192",	screen.Acolors[192],	DFT_COLOR("rgb:d7/ff/87")),
COLOR_RES("193",	screen.Acolors[193],	DFT_COLOR("rgb:d7/ff/af")),
COLOR_RES("194",	screen.Acolors[194],	DFT_COLOR("rgb:d7/ff/d7")),
COLOR_RES("195",	screen.Acolors[195],	DFT_COLOR("rgb:d7/ff/ff")),
COLOR_RES("196",	screen.Acolors[196],	DFT_COLOR("rgb:ff/00/00")),
COLOR_RES("197",	screen.Acolors[197],	DFT_COLOR("rgb:ff/00/5f")),
COLOR_RES("198",	screen.Acolors[198],	DFT_COLOR("rgb:ff/00/87")),
COLOR_RES("199",	screen.Acolors[199],	DFT_COLOR("rgb:ff/00/af")),
COLOR_RES("200",	screen.Acolors[200],	DFT_COLOR("rgb:ff/00/d7")),
COLOR_RES("201",	screen.Acolors[201],	DFT_COLOR("rgb:ff/00/ff")),
COLOR_RES("202",	screen.Acolors[202],	DFT_COLOR("rgb:ff/5f/00")),
COLOR_RES("203",	screen.Acolors[203],	DFT_COLOR("rgb:ff/5f/5f")),
COLOR_RES("204",	screen.Acolors[204],	DFT_COLOR("rgb:ff/5f/87")),
COLOR_RES("205",	screen.Acolors[205],	DFT_COLOR("rgb:ff/5f/af")),
COLOR_RES("206",	screen.Acolors[206],	DFT_COLOR("rgb:ff/5f/d7")),
COLOR_RES("207",	screen.Acolors[207],	DFT_COLOR("rgb:ff/5f/ff")),
COLOR_RES("208",	screen.Acolors[208],	DFT_COLOR("rgb:ff/87/00")),
COLOR_RES("209",	screen.Acolors[209],	DFT_COLOR("rgb:ff/87/5f")),
COLOR_RES("210",	screen.Acolors[210],	DFT_COLOR("rgb:ff/87/87")),
COLOR_RES("211",	screen.Acolors[211],	DFT_COLOR("rgb:ff/87/af")),
COLOR_RES("212",	screen.Acolors[212],	DFT_COLOR("rgb:ff/87/d7")),
COLOR_RES("213",	screen.Acolors[213],	DFT_COLOR("rgb:ff/87/ff")),
COLOR_RES("214",	screen.Acolors[214],	DFT_COLOR("rgb:ff/af/00")),
COLOR_RES("215",	screen.Acolors[215],	DFT_COLOR("rgb:ff/af/5f")),
COLOR_RES("216",	screen.Acolors[216],	DFT_COLOR("rgb:ff/af/87")),
COLOR_RES("217",	screen.Acolors[217],	DFT_COLOR("rgb:ff/af/af")),
COLOR_RES("218",	screen.Acolors[218],	DFT_COLOR("rgb:ff/af/d7")),
COLOR_RES("219",	screen.Acolors[219],	DFT_COLOR("rgb:ff/af/ff")),
COLOR_RES("220",	screen.Acolors[220],	DFT_COLOR("rgb:ff/d7/00")),
COLOR_RES("221",	screen.Acolors[221],	DFT_COLOR("rgb:ff/d7/5f")),
COLOR_RES("222",	screen.Acolors[222],	DFT_COLOR("rgb:ff/d7/87")),
COLOR_RES("223",	screen.Acolors[223],	DFT_COLOR("rgb:ff/d7/af")),
COLOR_RES("224",	screen.Acolors[224],	DFT_COLOR("rgb:ff/d7/d7")),
COLOR_RES("225",	screen.Acolors[225],	DFT_COLOR("rgb:ff/d7/ff")),
COLOR_RES("226",	screen.Acolors[226],	DFT_COLOR("rgb:ff/ff/00")),
COLOR_RES("227",	screen.Acolors[227],	DFT_COLOR("rgb:ff/ff/5f")),
COLOR_RES("228",	screen.Acolors[228],	DFT_COLOR("rgb:ff/ff/87")),
COLOR_RES("229",	screen.Acolors[229],	DFT_COLOR("rgb:ff/ff/af")),
COLOR_RES("230",	screen.Acolors[230],	DFT_COLOR("rgb:ff/ff/d7")),
COLOR_RES("231",	screen.Acolors[231],	DFT_COLOR("rgb:ff/ff/ff")),
COLOR_RES("232",	screen.Acolors[232],	DFT_COLOR("rgb:08/08/08")),
COLOR_RES("233",	screen.Acolors[233],	DFT_COLOR("rgb:12/12/12")),
COLOR_RES("234",	screen.Acolors[234],	DFT_COLOR("rgb:1c/1c/1c")),
COLOR_RES("235",	screen.Acolors[235],	DFT_COLOR("rgb:26/26/26")),
COLOR_RES("236",	screen.Acolors[236],	DFT_COLOR("rgb:30/30/30")),
COLOR_RES("237",	screen.Acolors[237],	DFT_COLOR("rgb:3a/3a/3a")),
COLOR_RES("238",	screen.Acolors[238],	DFT_COLOR("rgb:44/44/44")),
COLOR_RES("239",	screen.Acolors[239],	DFT_COLOR("rgb:4e/4e/4e")),
COLOR_RES("240",	screen.Acolors[240],	DFT_COLOR("rgb:58/58/58")),
COLOR_RES("241",	screen.Acolors[241],	DFT_COLOR("rgb:62/62/62")),
COLOR_RES("242",	screen.Acolors[242],	DFT_COLOR("rgb:6c/6c/6c")),
COLOR_RES("243",	screen.Acolors[243],	DFT_COLOR("rgb:76/76/76")),
COLOR_RES("244",	screen.Acolors[244],	DFT_COLOR("rgb:80/80/80")),
COLOR_RES("245",	screen.Acolors[245],	DFT_COLOR("rgb:8a/8a/8a")),
COLOR_RES("246",	screen.Acolors[246],	DFT_COLOR("rgb:94/94/94")),
COLOR_RES("247",	screen.Acolors[247],	DFT_COLOR("rgb:9e/9e/9e")),
COLOR_RES("248",	screen.Acolors[248],	DFT_COLOR("rgb:a8/a8/a8")),
COLOR_RES("249",	screen.Acolors[249],	DFT_COLOR("rgb:b2/b2/b2")),
COLOR_RES("250",	screen.Acolors[250],	DFT_COLOR("rgb:bc/bc/bc")),
COLOR_RES("251",	screen.Acolors[251],	DFT_COLOR("rgb:c6/c6/c6")),
COLOR_RES("252",	screen.Acolors[252],	DFT_COLOR("rgb:d0/d0/d0")),
COLOR_RES("253",	screen.Acolors[253],	DFT_COLOR("rgb:da/da/da")),
COLOR_RES("254",	screen.Acolors[254],	DFT_COLOR("rgb:e4/e4/e4")),
COLOR_RES("255",	screen.Acolors[255],	DFT_COLOR("rgb:ee/ee/ee")),

#endif /* included_256colres_h */
