/*
 * This header file was generated by ./88colres.pl
 */
/* $XTermId: 88colres.h,v 1.11 2007/06/12 00:44:21 tom Exp $ */

#ifndef included_88colres_h
#define included_88colres_h

COLOR_RES("16",	screen.Acolors[16],	DFT_COLOR("rgb:00/00/00")),
COLOR_RES("17",	screen.Acolors[17],	DFT_COLOR("rgb:00/00/8b")),
COLOR_RES("18",	screen.Acolors[18],	DFT_COLOR("rgb:00/00/cd")),
COLOR_RES("19",	screen.Acolors[19],	DFT_COLOR("rgb:00/00/ff")),
COLOR_RES("20",	screen.Acolors[20],	DFT_COLOR("rgb:00/8b/00")),
COLOR_RES("21",	screen.Acolors[21],	DFT_COLOR("rgb:00/8b/8b")),
COLOR_RES("22",	screen.Acolors[22],	DFT_COLOR("rgb:00/8b/cd")),
COLOR_RES("23",	screen.Acolors[23],	DFT_COLOR("rgb:00/8b/ff")),
COLOR_RES("24",	screen.Acolors[24],	DFT_COLOR("rgb:00/cd/00")),
COLOR_RES("25",	screen.Acolors[25],	DFT_COLOR("rgb:00/cd/8b")),
COLOR_RES("26",	screen.Acolors[26],	DFT_COLOR("rgb:00/cd/cd")),
COLOR_RES("27",	screen.Acolors[27],	DFT_COLOR("rgb:00/cd/ff")),
COLOR_RES("28",	screen.Acolors[28],	DFT_COLOR("rgb:00/ff/00")),
COLOR_RES("29",	screen.Acolors[29],	DFT_COLOR("rgb:00/ff/8b")),
COLOR_RES("30",	screen.Acolors[30],	DFT_COLOR("rgb:00/ff/cd")),
COLOR_RES("31",	screen.Acolors[31],	DFT_COLOR("rgb:00/ff/ff")),
COLOR_RES("32",	screen.Acolors[32],	DFT_COLOR("rgb:8b/00/00")),
COLOR_RES("33",	screen.Acolors[33],	DFT_COLOR("rgb:8b/00/8b")),
COLOR_RES("34",	screen.Acolors[34],	DFT_COLOR("rgb:8b/00/cd")),
COLOR_RES("35",	screen.Acolors[35],	DFT_COLOR("rgb:8b/00/ff")),
COLOR_RES("36",	screen.Acolors[36],	DFT_COLOR("rgb:8b/8b/00")),
COLOR_RES("37",	screen.Acolors[37],	DFT_COLOR("rgb:8b/8b/8b")),
COLOR_RES("38",	screen.Acolors[38],	DFT_COLOR("rgb:8b/8b/cd")),
COLOR_RES("39",	screen.Acolors[39],	DFT_COLOR("rgb:8b/8b/ff")),
COLOR_RES("40",	screen.Acolors[40],	DFT_COLOR("rgb:8b/cd/00")),
COLOR_RES("41",	screen.Acolors[41],	DFT_COLOR("rgb:8b/cd/8b")),
COLOR_RES("42",	screen.Acolors[42],	DFT_COLOR("rgb:8b/cd/cd")),
COLOR_RES("43",	screen.Acolors[43],	DFT_COLOR("rgb:8b/cd/ff")),
COLOR_RES("44",	screen.Acolors[44],	DFT_COLOR("rgb:8b/ff/00")),
COLOR_RES("45",	screen.Acolors[45],	DFT_COLOR("rgb:8b/ff/8b")),
COLOR_RES("46",	screen.Acolors[46],	DFT_COLOR("rgb:8b/ff/cd")),
COLOR_RES("47",	screen.Acolors[47],	DFT_COLOR("rgb:8b/ff/ff")),
COLOR_RES("48",	screen.Acolors[48],	DFT_COLOR("rgb:cd/00/00")),
COLOR_RES("49",	screen.Acolors[49],	DFT_COLOR("rgb:cd/00/8b")),
COLOR_RES("50",	screen.Acolors[50],	DFT_COLOR("rgb:cd/00/cd")),
COLOR_RES("51",	screen.Acolors[51],	DFT_COLOR("rgb:cd/00/ff")),
COLOR_RES("52",	screen.Acolors[52],	DFT_COLOR("rgb:cd/8b/00")),
COLOR_RES("53",	screen.Acolors[53],	DFT_COLOR("rgb:cd/8b/8b")),
COLOR_RES("54",	screen.Acolors[54],	DFT_COLOR("rgb:cd/8b/cd")),
COLOR_RES("55",	screen.Acolors[55],	DFT_COLOR("rgb:cd/8b/ff")),
COLOR_RES("56",	screen.Acolors[56],	DFT_COLOR("rgb:cd/cd/00")),
COLOR_RES("57",	screen.Acolors[57],	DFT_COLOR("rgb:cd/cd/8b")),
COLOR_RES("58",	screen.Acolors[58],	DFT_COLOR("rgb:cd/cd/cd")),
COLOR_RES("59",	screen.Acolors[59],	DFT_COLOR("rgb:cd/cd/ff")),
COLOR_RES("60",	screen.Acolors[60],	DFT_COLOR("rgb:cd/ff/00")),
COLOR_RES("61",	screen.Acolors[61],	DFT_COLOR("rgb:cd/ff/8b")),
COLOR_RES("62",	screen.Acolors[62],	DFT_COLOR("rgb:cd/ff/cd")),
COLOR_RES("63",	screen.Acolors[63],	DFT_COLOR("rgb:cd/ff/ff")),
COLOR_RES("64",	screen.Acolors[64],	DFT_COLOR("rgb:ff/00/00")),
COLOR_RES("65",	screen.Acolors[65],	DFT_COLOR("rgb:ff/00/8b")),
COLOR_RES("66",	screen.Acolors[66],	DFT_COLOR("rgb:ff/00/cd")),
COLOR_RES("67",	screen.Acolors[67],	DFT_COLOR("rgb:ff/00/ff")),
COLOR_RES("68",	screen.Acolors[68],	DFT_COLOR("rgb:ff/8b/00")),
COLOR_RES("69",	screen.Acolors[69],	DFT_COLOR("rgb:ff/8b/8b")),
COLOR_RES("70",	screen.Acolors[70],	DFT_COLOR("rgb:ff/8b/cd")),
COLOR_RES("71",	screen.Acolors[71],	DFT_COLOR("rgb:ff/8b/ff")),
COLOR_RES("72",	screen.Acolors[72],	DFT_COLOR("rgb:ff/cd/00")),
COLOR_RES("73",	screen.Acolors[73],	DFT_COLOR("rgb:ff/cd/8b")),
COLOR_RES("74",	screen.Acolors[74],	DFT_COLOR("rgb:ff/cd/cd")),
COLOR_RES("75",	screen.Acolors[75],	DFT_COLOR("rgb:ff/cd/ff")),
COLOR_RES("76",	screen.Acolors[76],	DFT_COLOR("rgb:ff/ff/00")),
COLOR_RES("77",	screen.Acolors[77],	DFT_COLOR("rgb:ff/ff/8b")),
COLOR_RES("78",	screen.Acolors[78],	DFT_COLOR("rgb:ff/ff/cd")),
COLOR_RES("79",	screen.Acolors[79],	DFT_COLOR("rgb:ff/ff/ff")),
COLOR_RES("80",	screen.Acolors[80],	DFT_COLOR("rgb:2e/2e/2e")),
COLOR_RES("81",	screen.Acolors[81],	DFT_COLOR("rgb:5c/5c/5c")),
COLOR_RES("82",	screen.Acolors[82],	DFT_COLOR("rgb:73/73/73")),
COLOR_RES("83",	screen.Acolors[83],	DFT_COLOR("rgb:8b/8b/8b")),
COLOR_RES("84",	screen.Acolors[84],	DFT_COLOR("rgb:a2/a2/a2")),
COLOR_RES("85",	screen.Acolors[85],	DFT_COLOR("rgb:b9/b9/b9")),
COLOR_RES("86",	screen.Acolors[86],	DFT_COLOR("rgb:d0/d0/d0")),
COLOR_RES("87",	screen.Acolors[87],	DFT_COLOR("rgb:e7/e7/e7")),

#endif /* included_88colres_h */
