# $XTermId: VTparse.def,v 1.76 2025/03/16 22:59:10 tom Exp $
#
# vile:confmode rs=lf
# -----------------------------------------------------------------------------
# this file is part of xterm
#
# Copyright 1996-2024,2025 by Thomas E. Dickey
# 
#                         All Rights Reserved
# 
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE ABOVE LISTED COPYRIGHT HOLDER(S) BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# 
# Except as contained in this notice, the name(s) of the above copyright
# holders shall not be used in advertising or otherwise to promote the
# sale, use or other dealings in this Software without prior written
# authorization.
# -----------------------------------------------------------------------------
#
# List of symbols that need to be defined for VTparse.h.  If you need to
# change any of the CASE_ macros, make the change here and rerun the command
# shown in VTparse.h.
#

CASE_GROUND_STATE
CASE_IGNORE
CASE_BELL
CASE_BS
CASE_CR
CASE_ESC
CASE_VMOT
CASE_TAB
CASE_SI
CASE_SO
CASE_SCR_STATE
CASE_SCS0_STATE
CASE_SCS1_STATE
CASE_SCS2_STATE
CASE_SCS3_STATE
CASE_ESC_IGNORE
CASE_ESC_DIGIT
CASE_ESC_SEMI
CASE_DEC_STATE
CASE_ICH
CASE_CUU
CASE_CUD
CASE_CUF
CASE_CUB
CASE_CUP
CASE_ED
CASE_EL
CASE_IL
CASE_DL
CASE_DCH
CASE_DA1
CASE_TRACK_MOUSE		# XTHIMOUSE
CASE_TBC
CASE_SET
CASE_RST
CASE_SGR
CASE_CPR
CASE_DECSTBM
CASE_DECREQTPARM
CASE_DECSET
CASE_DECRST
CASE_DECALN
CASE_GSETS
CASE_DECSC
CASE_DECRC
CASE_DECKPAM
CASE_DECKPNM
CASE_IND
CASE_NEL
CASE_HTS
CASE_RI
CASE_SS2
CASE_SS3
CASE_CSI_STATE
CASE_OSC
CASE_RIS
CASE_LS2
CASE_LS3
CASE_LS3R
CASE_LS2R
CASE_LS1R
CASE_PRINT
CASE_XTERM_SAVE			# XTSAVE
CASE_XTERM_RESTORE		# XTRESTORE
CASE_DECID
CASE_HP_MEM_LOCK		# HPMEMLOCK
CASE_HP_MEM_UNLOCK		# HPMEMUNLOCK
CASE_HP_BUGGY_LL		# HPBUGGYLL
CASE_HPA
CASE_VPA
CASE_XTERM_WINOPS		# XTWINOPS
CASE_ECH
CASE_CHT
CASE_CPL
CASE_CNL
CASE_CBT
CASE_SU
CASE_SD
CASE_S7C1T
CASE_S8C1T
CASE_ESC_SP_STATE
CASE_ENQ
CASE_DECSCL
CASE_DECSCA
CASE_DECSED
CASE_DECSEL
CASE_DCS
CASE_PM
CASE_SOS
CASE_ST
CASE_APC
CASE_EPA
CASE_SPA
CASE_CSI_QUOTE_STATE
CASE_DSR
CASE_ANSI_LEVEL_1
CASE_ANSI_LEVEL_2
CASE_ANSI_LEVEL_3
CASE_MC
CASE_DEC2_STATE
CASE_DA2
CASE_DEC3_STATE
CASE_DECRPTUI
CASE_VT52_CUP
CASE_REP
CASE_CSI_EX_STATE
CASE_DECSTR
CASE_DECDHL
CASE_DECSWL
CASE_DECDWL
CASE_DEC_MC
CASE_ESC_PERCENT
CASE_UTF8
CASE_CSI_TICK_STATE
CASE_DECELR
CASE_DECRQLP
CASE_DECEFR
CASE_DECSLE
CASE_CSI_IGNORE
CASE_VT52_IGNORE
CASE_VT52_FINISH
CASE_CSI_DOLLAR_STATE
CASE_DECCRA
CASE_DECERA
CASE_DECFRA
CASE_DECSERA
CASE_DECSACE
CASE_DECCARA
CASE_DECRARA
CASE_CSI_STAR_STATE
CASE_SET_MOD_FKEYS		# XTMODKEYS
CASE_SET_MOD_FKEYS0
CASE_HIDE_POINTER		# XTSMPOINTER
CASE_SCS1A_STATE
CASE_SCS2A_STATE
CASE_SCS3A_STATE
CASE_CSI_SPACE_STATE
CASE_DECSCUSR
CASE_XTERM_SM_TITLE		# XTSMTITLE
CASE_XTERM_RM_TITLE		# XTRMTITLE
CASE_DECSMBV
CASE_DECSWBV
CASE_DECLL
CASE_DECRQM
CASE_RQM
CASE_CSI_DEC_DOLLAR_STATE
CASE_SL
CASE_SR
CASE_DECDC
CASE_DECIC
CASE_DECBI
CASE_DECFI
CASE_DECRQCRA
CASE_HPR
CASE_VPR
CASE_ANSI_SC
CASE_ANSI_RC
CASE_ESC_COLON
CASE_SCS_PERCENT
CASE_GSETS_PERCENT
CASE_GRAPHICS_ATTRIBUTES	# XTSMGRAPHICS
CASE_DECRQPSR
CASE_DECSCPP
CASE_DECSNLS
CASE_CSI_HASH_STATE
CASE_XTERM_PUSH_SGR		# XTPUSHSGR
CASE_XTERM_REPORT_SGR		# XTREPORTSGR
CASE_XTERM_POP_SGR		# XTPOPSGR
CASE_XTERM_CHECKSUM		# XTCHECKSUM
CASE_GSETS3
CASE_GSETS5
CASE_SCS_DQUOTE
CASE_GSETS_DQUOTE
CASE_SCS_AMPRSND
CASE_GSETS_AMPRSND
CASE_REPORT_VERSION		# XTVERSION
CASE_XTERM_PUSH_COLORS		# XTPUSHCOLORS
CASE_XTERM_REPORT_COLORS	# XTREPORTCOLORS
CASE_XTERM_POP_COLORS		# XTPOPCOLORS
CASE_XTERM_SHIFT_ESCAPE		# XTSHIFTESCAPE
CASE_DECSSDT
CASE_DECSASD
CASE_XTERM_REPORT_MOD_FKEYS	# XTQMODKEYS
CASE_DECRQDE
CASE_DECRQUPSS
CASE_CSI_AMP_STATE
CASE_CSI_COMMA_STATE
CASE_DECAC
CASE_DECATC
CASE_DECTID
CASE_DECST8C
CASE_XTERM_TITLE_STACK		# XTTITLEPOS
CASE_SUB
CASE_CAN
CASE_SET_FMT_KEYS		# XTFMTKEYS
CASE_XTERM_REPORT_FMT_KEYS	# XTQFMTKEYS
